@@ -1,4 +1,4 @@
-// Copyright 2012-2021 The NATS Authors
+// Copyright 2012-2023 The NATS Authors
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
 // You may obtain a copy of the License at
@@ -89,6 +89,19 @@ func ExampleConn_Subscribe() {
 	})
 }
 
+func ExampleConn_ForceReconnect() {
+	nc, _ := nats.Connect(nats.DefaultURL)
+	defer nc.Close()
+
+	nc.Subscribe("foo", func(m *nats.Msg) {
+		fmt.Printf("Received a message: %s\n", string(m.Data))
+	})
+
+	// Reconnect to the server.
+	// the subscription will be recreated after the reconnect.
+	nc.ForceReconnect()
+}
+
 // This Example shows a synchronous subscriber.
 func ExampleConn_SubscribeSync() {
 	nc, _ := nats.Connect(nats.DefaultURL)
@@ -214,106 +227,6 @@ func ExampleConn_Close() {
 	nc.Close()
 }
 
-// Shows how to wrap a Conn into an EncodedConn
-func ExampleNewEncodedConn() {
-	nc, _ := nats.Connect(nats.DefaultURL)
-	c, _ := nats.NewEncodedConn(nc, "json")
-	c.Close()
-}
-
-// EncodedConn can publish virtually anything just
-// by passing it in. The encoder will be used to properly
-// encode the raw Go type
-func ExampleEncodedConn_Publish() {
-	nc, _ := nats.Connect(nats.DefaultURL)
-	c, _ := nats.NewEncodedConn(nc, "json")
-	defer c.Close()
-
-	type person struct {
-		Name    string
-		Address string
-		Age     int
-	}
-
-	me := &person{Name: "derek", Age: 22, Address: "85 Second St"}
-	c.Publish("hello", me)
-}
-
-// EncodedConn's subscribers will automatically decode the
-// wire data into the requested Go type using the Decode()
-// method of the registered Encoder. The callback signature
-// can also vary to include additional data, such as subject
-// and reply subjects.
-func ExampleEncodedConn_Subscribe() {
-	nc, _ := nats.Connect(nats.DefaultURL)
-	c, _ := nats.NewEncodedConn(nc, "json")
-	defer c.Close()
-
-	type person struct {
-		Name    string
-		Address string
-		Age     int
-	}
-
-	c.Subscribe("hello", func(p *person) {
-		fmt.Printf("Received a person! %+v\n", p)
-	})
-
-	c.Subscribe("hello", func(subj, reply string, p *person) {
-		fmt.Printf("Received a person on subject %s! %+v\n", subj, p)
-	})
-
-	me := &person{Name: "derek", Age: 22, Address: "85 Second St"}
-	c.Publish("hello", me)
-}
-
-// BindSendChan() allows binding of a Go channel to a nats
-// subject for publish operations. The Encoder attached to the
-// EncodedConn will be used for marshaling.
-func ExampleEncodedConn_BindSendChan() {
-	nc, _ := nats.Connect(nats.DefaultURL)
-	c, _ := nats.NewEncodedConn(nc, "json")
-	defer c.Close()
-
-	type person struct {
-		Name    string
-		Address string
-		Age     int
-	}
-
-	ch := make(chan *person)
-	c.BindSendChan("hello", ch)
-
-	me := &person{Name: "derek", Age: 22, Address: "85 Second St"}
-	ch <- me
-}
-
-// BindRecvChan() allows binding of a Go channel to a nats
-// subject for subscribe operations. The Encoder attached to the
-// EncodedConn will be used for un-marshaling.
-func ExampleEncodedConn_BindRecvChan() {
-	nc, _ := nats.Connect(nats.DefaultURL)
-	c, _ := nats.NewEncodedConn(nc, "json")
-	defer c.Close()
-
-	type person struct {
-		Name    string
-		Address string
-		Age     int
-	}
-
-	ch := make(chan *person)
-	c.BindRecvChan("hello", ch)
-
-	me := &person{Name: "derek", Age: 22, Address: "85 Second St"}
-	c.Publish("hello", me)
-
-	// Receive the publish directly on a channel
-	who := <-ch
-
-	fmt.Printf("%v says hello!\n", who)
-}
-
 func ExampleJetStream() {
 	nc, err := nats.Connect("localhost")
 	if err != nil {
@@ -612,7 +525,7 @@ func ExampleSubOpt() {
 		nats.MaxDeliver(1),
 		// Activate Flow control algorithm from the server.
 		nats.EnableFlowControl(),
-		// Track heartbeats from the server fro missed sequences.
+		// Track heartbeats from the server for missed sequences.
 		nats.IdleHeartbeat(500*time.Millisecond),
 	)
 
