@@ -39,6 +39,14 @@ type jsServer struct {
 	restart sync.Mutex
 }
 
+// Restart can be used to start again a server
+// using the same listen address as before.
+func (srv *jsServer) Restart() {
+	srv.restart.Lock()
+	defer srv.restart.Unlock()
+	srv.Server = natsserver.RunServer(srv.myopts)
+}
+
 // Dumb wait program to sync on callbacks, etc... Will timeout
 func Wait(ch chan bool) error {
 	return WaitTime(ch, 5*time.Second)
@@ -74,15 +82,6 @@ func NewConnection(t *testing.T, port int) *nats.Conn {
 	return nc
 }
 
-// NewEConn
-func NewEConn(t *testing.T) *nats.EncodedConn {
-	ec, err := nats.NewEncodedConn(NewDefaultConnection(t), nats.DEFAULT_ENCODER)
-	if err != nil {
-		t.Fatalf("Failed to create an encoded connection: %v\n", err)
-	}
-	return ec
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 // Running nats server in separate Go routines
 ////////////////////////////////////////////////////////////////////////////////
